#!/usr/bin/env python
# encoding: utf-8

"""MoodleFUSE initialization
"""

import os
import errno

from moodlefuse.filesystem import Filesystem
from moodlefuse.core import setup, config
from moodlefuse.services import USERS


MOODLEFUSE_DATABASE_FILE = 'moodlefuse.sqlite'
MOODLEFUSE_CONFIG_FILE = 'moodlefuse.conf'
MOODLEFUSE_HIDDEN_FOLDER = '.moodlefuse'
DATABASE_CONF = 'alembic.ini'
MOODLEFUSE_CACHE = 'cache'


class MoodleFuse(object):

    def __init__(self, settings=None, testing=None):
        setup(settings)
        self._create_filesystem_root()
        Filesystem(config['LOCAL_MOODLE_FOLDER'], testing)

    def _create_filesystem_root(self):
        moodle_fs_path = config['LOCAL_MOODLE_FOLDER']
        try:
            os.makedirs(moodle_fs_path)
        except OSError as e:
            if e.errno is not errno.EEXIST:
                raise e



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle errors relating to resource incidents.
"""

from moodlefuse.moodle.exception import MoodleException


class UnableToObtainResourceList(MoodleException):
    def __init__(self):
        debug_info = "Failed to obtain resource list from Moodle"
        super(UnableToObtainResourceList, self).__init__(debug_info)


class UnableToUploadResource(MoodleException):
    def __init__(self):
        debug_info = "Failed to upload resource"
        super(UnableToUploadResource, self).__init__(debug_info)


class UnableToGetLocalFile(MoodleException):
    def __init__(self):
        debug_info = "Failed to get the local resource"
        super(UnableToGetLocalFile, self).__init__(debug_info)


class UnableToDownloadResource(MoodleException):
    def __init__(self):
        debug_info = "Failed to download resource"
        super(UnableToDownloadResource, self).__init__(debug_info)


class UnableToRenameFile(MoodleException):
    def __init__(self):
        debug_info = "Unable to rename moodle file"
        super(UnableToRenameFile, self).__init__(debug_info)


class UnableToModifyFile(MoodleException):
    def __init__(self):
        debug_info = "Unable to modify moodle file"
        super(UnableToModifyFile, self).__init__(debug_info)


class UnableToAddFile(MoodleException):
    def __init__(self):
        debug_info = "Unable to add moodle file"
        super(UnableToAddFile, self).__init__(debug_info)


class UnableToRemoveFile(MoodleException):
    def __init__(self):
        debug_info = "Unable to modify moodle file"
        super(UnableToRemoveFile, self).__init__(debug_info)



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle assignment Moodle actions, it extends MoodleHandler
   and therefor is an observer of Moodle.
"""

from moodlefuse.moodle.assignments.assignment_grader import AssignmentGrader
from moodlefuse.moodle.assignments.assignment_parser import AssignmentParser
from moodlefuse.moodle.handler import MoodleHandler
from moodlefuse.moodle import assignments


class AssignmentHandler(MoodleHandler):

    def __init__(self, emulator, js_emulator):
        super(self.__class__, self).__init__(emulator, js_emulator)
        self.grader = AssignmentGrader()
        self.parser = AssignmentParser()

    def get_assignment_info_as_array(self):
        return [assignments.SUBMISSIONS_FILENAME, assignments.GRADES_FILENAME]

    def get_grading_url(self, assignment_url):
        if 'grading' not in assignment_url:
            assignment_url = assignment_url + assignments.GRADING_URL_EXTENSION

        return assignment_url

    def get_assignment_content(self, assignment_url):
        assignment_url = self.get_grading_url(assignment_url)
        return self.moodle.follow_link(assignment_url)

    def get_assignment_submissions(self, assignment_url):
        self.get_assignment_content(assignment_url)
        assignment_submissions = self.moodle.filter_assignment_submissions()
        return self.parser.get_all_submission_names(assignment_submissions)

    def populate_grading_form(self, location, grading_info):
        csv = self.grader.format_csv(location)
        csv = self.grader.add_user_information(csv, grading_info)
        return csv

    def get_grade_info_from_url(self, assignment_url):
        self.get_assignment_content(assignment_url)
        assignment_submissions = self.moodle.unfilter_assignment_submissions()
        return self.get_grading_info(assignment_submissions)

    def get_grading_info(self, assignment_submissions):
        names = self.parser.get_all_student_names(assignment_submissions)
        emails = self.parser.get_all_student_emails(assignment_submissions)
        grades = self.parser.get_all_student_grades(assignment_submissions)
        return (names, emails, grades)

    def modify_grades(self, location, assignment_url):
        old_grading_info = self.get_grade_info_from_url(assignment_url)
        grades = self.grader.get_modified(location, old_grading_info)
        self.moodle.follow_link_with_js(self.get_grading_url(assignment_url))
        self.moodle.modify_assignment_grades(grades)

    def get_grades_csv(self, location, assignment_url):
        grading_info = self.get_grade_info_from_url(assignment_url)
        return self.populate_grading_form(location, grading_info)



#!/usr/bin/env python
# encoding: utf-8

from stat import S_IFDIR, S_IREAD, S_IWRITE
from moodlefuse.filesystem.files.file import File


class Directory(File):

    def __init__(self):
        super(Directory, self).__init__()
        self.attrs['st_mode'] = (S_IFDIR | S_IREAD | S_IWRITE)
        self.attrs['st_size'] = 4096

    def get_aattrs(self):
        return self.attrs



#!/usr/bin/env python
# encoding: utf-8


class TestUtils(object):

    @staticmethod
    def lists_are_equal(list1, list2):
        return sorted(list1) == sorted(list2)

    @staticmethod
    def lists_are_not_equal(list1, list2):
        return sorted(list1) != sorted(list2)


#!/usr/bin/env python
# encoding: utf-8

import os
import shutil

from tests.utils import TestUtils
from unittest import TestCase
from moodlefuse import MoodleFuse
from tests.data import settings
from moodlefuse.filesystem.file_operations import FileOperationOverrider


class MoodleFuseTestCase(TestCase):
    pass

class MoodleFuseAppTestCase(MoodleFuseTestCase):

    @classmethod
    def setUpClass(cls):
        cls.filesystem_root = os.path.join(
            os.path.expanduser('~'),
            settings.LOCAL_MOODLE_FOLDER
        )
        if not os.path.exists(cls.filesystem_root):
            os.mkdir(cls.filesystem_root)

        if not os.path.exists(settings.DOWNLOADS):
            os.mkdir(settings.DOWNLOADS)

        cls.app = MoodleFuse(settings, True)
        cls.ops = FileOperationOverrider(cls.filesystem_root)

    @classmethod
    def tearDownClass(cls):
        cls.ops.end_operations()

        if os.path.exists(cls.filesystem_root):
            os.rmdir(cls.filesystem_root)

        if os.path.exists(settings.DOWNLOADS):
            shutil.rmtree(settings.DOWNLOADS)

    def setUp(self):
        self.utils = TestUtils()

    def tearDown(self):
        pass



#!/usr/bin/env python
# encoding: utf-8

import os

from moodlefuse.models.users import User
from moodlefuse.services import USERS


def get_cache_path_based_on_location(location):
    config_folder = os.path.join(os.path.expanduser('~'), '.moodlefuse')
    cache_base = os.path.join(config_folder, 'cache')
    return cache_base + '/' + "_".join(location)


def get_password_for_user(username):
    return USERS.get(
        User,
        username
    ).password



from tests import MoodleFuseAppTestCase
from tests.categories.data.categories_results import true_categories, false_categories
from fuse import FuseOSError

class CourseCategoriesTestCase(MoodleFuseAppTestCase):

    def list_course_categories_test(self):
        files = list(self.ops.readdir(
            self.filesystem_root + '/testcourse', None))

        assert self.utils.lists_are_equal(files, true_categories)

    def list_course_categories_invalid_categories_test(self):
        files = list(self.ops.readdir(
            self.filesystem_root + '/testcourse', None))

        assert self.utils.lists_are_not_equal(files, false_categories)

    def enter_course_category_invalid_course_category_test(self):
        self.assertRaises(FuseOSError, lambda:
                self.ops.access(
                    self.filesystem_root + '/testcourse/invalidcategory', None
                )
        )


#!/usr/bin/env python
# encoding: utf-8

GRADES_FILENAME = 'grades.csv'
SUBMISSIONS_FILENAME = 'Submissions'
SUBMISSION_LISTING_END = '_submission'
GRADING_URL_EXTENSION = '&action=grading'
GRADING_SLASH_DELIMINATOR = '/'
GRADING_BLANK_GRADE = '-'
GRADING_HEADER_NAME = 'Name'
GRADING_HEADER_EMAIL = 'Email'
GRADING_HEADER_GRADE = 'Grade'



#!/usr/bin/env python
# encoding: utf-8

"""Class to spin up a headless browser to handle Moodle interaction
"""

from mechanize import Browser, CookieJar
from bs4 import BeautifulSoup

from moodlefuse.exception import throws_moodlefuse_error
from moodlefuse.moodle.emulator.emulator import Emulator
from moodlefuse.moodle.resources import resource_errors
from moodlefuse.moodle.courses import course_errors
from moodlefuse.moodle import exception, attributes
from moodlefuse.core import config
from moodlefuse import moodle


class CoreEmulator(Emulator):

    def __init__(self, username, password):
        super(CoreEmulator, self).__init__(username, password)
        self.setup_emulator()

    def setup_emulator(self):
        self.browser = Browser()
        self.browser.set_handle_robots(False)
        self.browser.addheaders = moodle.USER_AGENT
        self.cookiejar = CookieJar()
        self.browser.set_cookiejar(self.cookiejar)

    @throws_moodlefuse_error(exception.LoginException)
    def login(self):
        self.open_login_page(self.browser.open)
        self.browser.select_form(
            predicate=lambda form: form.attrs.get('id') == attributes.LOGIN
        )
        self.browser.form.set_value(self.username, name='username')
        self.browser.form.set_value(self.password, name='password')
        resp = self.browser.submit()

        if resp.geturl().endswith(moodle.LOGIN_LOCATION):
            raise Exception

    @throws_moodlefuse_error(resource_errors.UnableToDownloadResource)
    def download(self, destination, source):
        source = str(source)
        if not source.startswith('http://') and not source.startswith('file://'):
            source = config['TEST_DATA'] + '/' + source

        self.browser.retrieve(source, destination)

    def open_link(self, url):
        response = self.browser.open(url)
        return BeautifulSoup(response.read())

    def check_form_checkbox(self, checkboxname):
        self.browser.find_control(checkboxname).items[0].selected = True

    def uncheck_form_checkbox(self, checkboxname):
        self.browser.find_control(checkboxname).items[0].selected = False

    def add_form_content(self, inputname, content):
        self.browser.form.set_value(content, name=inputname)

    def close_form(self):
        self.browser.submit()

    def set_form_to_first_form(self):
        self.browser.select_form(nr=0)

    def set_form_to_form_with_control_value(self, value):
        for form in self.browser.forms():
            for control in form.controls:
                if control.value == value:
                    self.browser.form = form

    @throws_moodlefuse_error(exception.UnableToToggleEditing)
    def turn_course_editing_on(self):
        self.set_form_to_form_with_control_value(moodle.EDIT_ON_MOODLE_BUTTON_TEXT)
        response = self.browser.submit()
        return BeautifulSoup(response.read())

    def _setup_assignments_for_parsing(self, submission_filter):
        self.set_form_to_form_with_control_value('Save and update table')
        self.browser.form["filter"] = [submission_filter]
        self.browser.form["perpage"] = ["100"]
        self.uncheck_form_checkbox('quickgrading')
        response = self.browser.submit()
        return BeautifulSoup(response.read())

    def filter_assignment_submissions(self):
        return self._setup_assignments_for_parsing("submitted")

    def unfilter_assignment_submissions(self):
        return self._setup_assignments_for_parsing("")

    @throws_moodlefuse_error(exception.UnableToToggleEditing)
    def turn_course_editing_off(self):
        self.set_form_to_form_with_control_value(moodle.EDIT_OFF_MOODLE_BUTTON_TEXT)
        response = self.browser.submit()
        return BeautifulSoup(response.read())

    @throws_moodlefuse_error(course_errors.InvalidMoodleIndex)
    def get_courses(self):
        return self.open_link(config['MOODLE_INDEX_ADDRESS'])

    @throws_moodlefuse_error(course_errors.UnableToObtainCategoryList)
    def get_course_categories(self, url):
        return self.open_link(url)

    @throws_moodlefuse_error(resource_errors.UnableToObtainResourceList)
    def get_course_resource_names(self, url):
        return self.open_link(url)

    def close(self):
        self.browser.close()



#!/usr/bin/env python
# encoding: utf-8

import os

from moodlefuse.helpers import get_cache_path_based_on_location
from moodlefuse.filesystem.files.file import File
from moodlefuse.core import config


class CacheFile(File):

    def __init__(self, cache_path=None):
        super(CacheFile, self).__init__()
        if cache_path is not None:
            self.path = cache_path
            self.set_attrs()

    def set_attrs(self):
        if os.path.isfile(self.path):
            st = os.lstat(self.path)
            self.attrs = dict(
                (key, getattr(st, key)) for key in (
                    'st_atime',
                    'st_ctime',
                    'st_gid',
                    'st_mode',
                    'st_mtime',
                    'st_nlink',
                    'st_size',
                    'st_uid'
                )
            )

    def create_file(self, location, content=None):
        if 'DOWNLOADS' in config:
            cache_path = config['DOWNLOADS'] + '/testfile'
        else:
            cache_path = get_cache_path_based_on_location(location)
        with open(cache_path, 'w') as cache_file:
            if content is not None:
                cache_file.write(content)
            cache_file.close()
            self.path = cache_path
            self.set_attrs()
            return cache_path



from tests import MoodleFuseAppTestCase
from tests.assignments.data.assignments_results import \
    true_assignment_submissions, grades_header, \
    true_assignment_info, false_assignment_submissions
from fuse import FuseOSError
from data.settings import DOWNLOADS

import os
import csv

class AssignmentsTestCase(MoodleFuseAppTestCase):

    def list_assignment_info_test(self):
        files = list(self.ops.readdir(
            self.filesystem_root + '/testcourse/e/test', None))

        assert self.utils.lists_are_equal(files, true_assignment_info)

    def list_assignment_info__invalid_info_test(self):
        files = list(self.ops.readdir(
            self.filesystem_root + '/testcourse/e/test', None))

        assert self.utils.lists_are_not_equal(files, false_assignment_submissions)


    def list_assignment_submissions_test(self):
        files = list(self.ops.readdir(
            self.filesystem_root + '/testcourse/e/test/Submissions', None))

        assert self.utils.lists_are_equal(files, true_assignment_submissions)

    def list_assignment_submissions_invalid_submissions_test(self):
        files = list(self.ops.readdir(
            self.filesystem_root + '/testcourse/e/test/submissions', None))

        assert self.utils.lists_are_not_equal(files, false_assignment_submissions)

    def enter_assignment_invalid_assignment_test(self):
        self.assertRaises(FuseOSError, lambda:
                self.ops.access(
                    self.filesystem_root + '/testcourse/e/invalidassignment', None
                )
        )

    def open_assignment_grading_form_test(self):
        status = self.ops.open(
            self.filesystem_root + '/testcourse/e/test/grades.csv', os.O_RDONLY)

        self.assertTrue(status > 1)

    def assignment_grades_form_cached_test(self):
        self.ops.open(
            self.filesystem_root + '/testcourse/e/test/grades.csv', os.O_RDONLY)

        self.assertTrue(os.path.exists(
                os.path.join(DOWNLOADS, 'grades.csv')
            )
        )

    def assignment_grading_formatted_test(self):
        self.ops.open(
            self.filesystem_root + '/testcourse/e/test/grades.csv', os.O_RDONLY)

        csv_file = os.path.join(DOWNLOADS, 'grades.csv')
        with open(csv_file, 'r') as file:
            reader = csv.reader(file)
            self.assertTrue(
                self.utils.lists_are_equal(
                    list(reader)[0], grades_header
                )
            )



#!/usr/bin/env python
# encoding: utf-8

"""Derived class to parse Moodle course items, it parses
   Moodle and therefor extends off of Parser
"""

from moodlefuse.moodle.parser import Parser
from moodlefuse.moodle import assignments
from moodlefuse.moodle import attributes


class AssignmentParser(Parser):

    def __init__(self):
        super(AssignmentParser, self).__init__()

    def get_all_submission_names(self, assignment_content):
        submission_names = []
        names = self.get_all_student_names(assignment_content)
        for name in names:
            name = name.replace(" ", "_")
            name = name + assignments.SUBMISSION_LISTING_END
            submission_names.append(name)

        return submission_names

    def get_all_student_names(self, assignment_content):
        return self.get_items_from_table(
            assignment_content, attributes.THIRD_CELL
        )

    def get_all_student_emails(self, assignment_content):
        return self.get_items_from_table(
            assignment_content, attributes.THIRD_CELL_EMAIL
        )

    def get_all_student_grades(self, assignment_content):
        return self.get_items_from_table(
            assignment_content, attributes.FIFTH_CELL, self.remove_special_chars
        )

    def get_items_from_table(self, html, tdclass, convert_function=None):
        items = []
        item_list = self.scraper.get_html_items_with_tdclass(html, tdclass)
        for html_item in item_list:
            item = html_item.get_text()
            if item != '':
                if convert_function is not None:
                    item = convert_function(item)
                items.append(item)

        return self.remove_unicode(items)



#!/usr/bin/env python
# encoding: utf-8

"""Class to translate filesystem events to Moodle actions
"""

import os

from moodlefuse.moodle.assignments.assignment_handler import AssignmentHandler
from moodlefuse.moodle.resources.resource_handler import ResourceHandler
from moodlefuse.moodle.emulator.js_enabled_emulator import JsEmulator
from moodlefuse.moodle.courses.course_handler import CourseHandler
from moodlefuse.moodle.emulator.core_emulator import CoreEmulator
from moodlefuse.helpers import get_cache_path_based_on_location
from moodlefuse.filesystem.remote_handler import RemoteHandler
from moodlefuse.filesystem.files.cache_file import CacheFile
from moodlefuse.filesystem.files.directory import Directory
from moodlefuse.filesystem.path_parser import PathParser
from moodlefuse.moodle import assignments
from moodlefuse.core import config


class FileSystemTranslator(object):

    def __init__(self):
        self.emulator = CoreEmulator(config['USERNAME'], config['PASSWORD'])
        self.js_emulator = JsEmulator(config['USERNAME'], config['PASSWORD'])
        self.emulator.login()
        self.js_emulator.login()
        course = CourseHandler(self.emulator, self.js_emulator)
        resource = ResourceHandler(self.emulator, self.js_emulator)
        assignment = AssignmentHandler(self.emulator, self.js_emulator)
        self.remote_handler = RemoteHandler(course, resource, assignment)

    def close_browsers(self):
        self.emulator.close()
        self.js_emulator.close()

    def open_file(self, path):
        location = PathParser.get_position_in_filesystem_as_array(path)
        if PathParser.is_file(location):
            moodle_url = self.remote_handler.get_remote_file_path(location)
            if moodle_url is None:
                return CacheFile().create_file(location)
            else:
                return self.remote_handler.download_updated_file(location, moodle_url)
        elif PathParser.is_assignment(location):
            return self.remote_handler.get_remote_grading_csv(location)

    def rename_file(self, old_path, new_path):
        old_location = PathParser.get_position_in_filesystem_as_array(old_path)
        new_location = PathParser.get_position_in_filesystem_as_array(new_path)
        if PathParser.is_file(old_location):
            self.remote_handler.rename_resource(old_location, new_location)

    def modify_file(self, path):
        location = PathParser.get_position_in_filesystem_as_array(path)
        cache_path = get_cache_path_based_on_location(location)
        self.remote_handler.modify_resource(location, cache_path)

    def modify_grades(self, path):
        location = PathParser.get_position_in_filesystem_as_array(path)
        self.remote_handler.modify_grades(location)

    def create_file(self, path):
        location = PathParser.get_position_in_filesystem_as_array(path)
        if PathParser.is_file(location):
            cache_path = get_cache_path_based_on_location(location)
            CacheFile(cache_path).create_file(location, ' ')
            self.remote_handler.add_resource(location, cache_path)
            return cache_path

    def make_directory(self, path):
        location = PathParser.get_position_in_filesystem_as_array(path)
        if PathParser.is_in_course_categorie(location):
            self.remote_handler.add_category(location)

    def use_cache_file_or_get_update_file(self, location, cache_path):
        if not os.path.isfile(cache_path):
            if self.is_assignment_grading_form(location):
                return self.remote_handler.get_remote_grading_csv(location)
            else:
                moodle_url = self.remote_handler.get_remote_file_path(location)
                return self.remote_handler.download_updated_file(location, moodle_url)
        return cache_path

    def is_assignment_grading_form(self, location):
        if PathParser.is_assignment(location):
            return location[3] == assignments.GRADES_FILENAME

        return False

    def remove_resource(self, path):
        location = PathParser.get_position_in_filesystem_as_array(path)
        if PathParser.is_file(location):
            return self.remote_handler.remove_resource(location)

    def file_is_assignment(self, location):
        return self.remote_handler.is_valid_assignment(location)

    def represent_as_directory(self, location):
        return self.file_is_assignment(location) and not \
            self.is_assignment_grading_form(location)

    def represent_as_file(self, location):
        return (PathParser.is_file(location) or
                PathParser.is_assignment(location) or
                PathParser.is_assignment_submission(location)) and not \
            self.represent_as_directory(location)

    def get_file_attributes(self, path):
        location = PathParser.get_position_in_filesystem_as_array(path)
        if self.represent_as_file(location):
            cache_path = get_cache_path_based_on_location(location)
            self.use_cache_file_or_get_update_file(location, cache_path)
            return CacheFile(cache_path).get_attrs()
        else:
            return Directory().get_aattrs()

    def get_directory_contents_based_on_path(self, path):
        location = PathParser.get_position_in_filesystem_as_array(path)
        dirents = ['.', '..']
        handle = {
            PathParser.is_in_root(location): self.remote_handler.get_remote_courses,
            PathParser.is_in_course(location): self.remote_handler.get_remote_categories,
            PathParser.is_in_course_categorie(location): self.remote_handler.get_remote_resourse_names,
            PathParser.is_file(location): self.remote_handler.get_remote_assignment_info,
            PathParser.is_assignment(location): self.remote_handler.get_remote_assignment_names,
        }

        if True in handle:
            dirents.extend((handle.get(True))(location))

        return dirents

    def path_exists_in_moodle(self, path):
        location = PathParser.get_position_in_filesystem_as_array(path)
        handle = {
            PathParser.is_in_root(location): self.remote_handler.is_valid_root,
            PathParser.is_in_course(location): self.remote_handler.is_valid_course,
            PathParser.is_in_course_categorie(location): self.remote_handler.is_valid_category,
            PathParser.is_file(location): self.remote_handler.is_valid_resource,
            PathParser.is_assignment(location): self.remote_handler.is_valid_assignment_info,
            PathParser.is_assignment_submission(location): self.remote_handler.is_valid_assignment_submission
        }

        if True in handle:
            return (handle.get(True))(location)
        else:
            return False



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle resource Moodle actions, it extends MoodleHandler
   and therefor is an observer of Moodle.
"""

from moodlefuse.moodle.resources.resource_parser import ResourceParser
from moodlefuse.filesystem.files.cache_file import CacheFile
from moodlefuse.moodle.handler import MoodleHandler
from moodlefuse.moodle import resources


class ResourceHandler(MoodleHandler):

    def __init__(self, emulator, js_emulator):
        super(self.__class__, self).__init__(emulator, js_emulator)
        self.parser = ResourceParser()

    def get_file_names_as_array(self, category_contents):
        if category_contents is None:
            return []

        return self.parser.parse_course_resources(category_contents)

    def get_file_path(self, category_contents, filename):
        if category_contents is None:
            return []

        return self.parser.parse_course_resource_url(
            category_contents,
            filename
        )

    def get_assignment_url(self, category_contents, assignment):
        return self.parser.parse_course_assignment_url(
            category_contents,
            assignment
        )

    def is_assignment(self, category_contents, filename):
        return self.parser.parse_course_assignment_url(
            category_contents,
            filename
        ) is not None

    def download_resource(self, location, moodle_url):
        cache_path = CacheFile().create_file(location)

        if 'forcedownload' not in moodle_url:
            moodle_download_url = moodle_url + resources.DOWNLOAD_URL_EXTENSION
        else:
            moodle_download_url = moodle_url

        self.moodle.download_resources(cache_path, moodle_download_url)
        return cache_path

    def add_resource(self, resource_path, category, resource_name):
        self.moodle.add_new_resource(category, resource_name, resource_path)

    def modify_resource(self, resource_path, category, resource_name):
        self.moodle.modify_existing_resource(category, resource_name, resource_path)

    def remove_resource(self, category, resource_name):
        self.moodle.remove_existing_resource(category, resource_name)

    def rename_resource(self, category, old_resource_name, new_resource_name):
        self.moodle.rename_existing_resource(category, old_resource_name, new_resource_name)



#!/usr/bin/env python
# encoding: utf-8
from moodlefuse.moodle.resources.resource_errors import UnableToDownloadResource
from tests.resources.data.resources_results import true_resources, false_resources
from tests import MoodleFuseAppTestCase
from tests.data import settings
from fuse import FuseOSError

import os


class ResourcesTestCase(MoodleFuseAppTestCase):

    def list_resources_test(self):
        files = list(
            self.ops.readdir(
                self.filesystem_root + '/testcourse/week_1', None))

        assert self.utils.lists_are_equal(files, true_resources)

    def list_resources_invalid_resources_test(self):
        files = list(
            self.ops.readdir(
                self.filesystem_root + '/testcourse/week_1', None))

        assert self.utils.lists_are_not_equal(files, false_resources)

    def download_resource_test(self):
        self.ops.open(
            self.filesystem_root + '/testcourse/week_1/t', os.O_RDONLY)

        self.assertTrue(os.path.exists(settings.DOWNLOADS + '/testfile'))
        os.remove(settings.DOWNLOADS + '/testfile')

    def get_resource_attributes_test(self):
        attributes = self.ops.getattr(
            self.filesystem_root + '/testcourse/week_1/r'
        )

        self.assertTrue(
            (attributes.has_key(key)) for key in (
                'st_atime',
                'st_ctime',
                'st_gid',
                'st_mode',
                'st_mtime',
                'st_nlink',
                'st_size',
                'st_uid'
            )
        )

    def get_resource_attributes_invalid_resource_test(self):
        self.assertRaises(FuseOSError, lambda:
                self.ops.getattr(
                    self.filesystem_root + '/testcourse/week_1/invalidresource'
                )
        )


#!/usr/bin/env python
# encoding: utf-8

from moodlefuse.exception import throws_moodlefuse_error
from moodlefuse.moodle import exception
from moodlefuse.core import config
from moodlefuse import moodle


class Emulator(object):

    def __init__(self, username, password):
        self.username = username
        self.password = password

    def setup_emulator(self):
        pass

    def login(self):
        pass

    def turn_course_editing_on(self):
        pass

    def turn_course_editing_off(self):
        pass

    def close(self):
        pass

    @throws_moodlefuse_error(exception.NotFoundException)
    def open_login_page(self, open_url_function):
        if not config['MOODLE_WEB_ADDRESS'].endswith('php') and not config['MOODLE_WEB_ADDRESS'].endswith('html'):
            MOODLE_LOGIN_URL = config['MOODLE_WEB_ADDRESS'] + moodle.LOGIN_LOCATION
        else:
            MOODLE_LOGIN_URL = config['MOODLE_WEB_ADDRESS']

        open_url_function(MOODLE_LOGIN_URL)



#!/usr/bin/env python
# encoding: utf-8

from functools import wraps


LOGIN_LOCATION = '/login/index.php'
EDIT_ON_MOODLE_BUTTON_TEXT = 'Turn editing on'
EDIT_OFF_MOODLE_BUTTON_TEXT = 'Turn editing off'
USER_AGENT = [
    ('User-Agent',
     'Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 \
     (KHTML, like Gecko) Chrome/34.0.1847.132 Safari/537.36'),
    ('Accept', 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8'),
    ('Cache-Control', 'max-age=0'),
    ('Accept-Language', 'en-GB,en-US;q=0.8,en;q=0.6'),
    ('Connection', 'keep-alive')
]


def requires_editing_moodle_js():
    def inner(f):
        def wrapped(*args):
            wrapped_class = args[0]
            wrapped_class.js_emulator.turn_course_editing_on()
            result = f(*args)
            wrapped_class.js_emulator.turn_course_editing_off()
            return result
        return wraps(f)(wrapped)
    return inner


def requires_valid_cookie():
    def inner(f):
        def wrapped(*args):
            wrapped_class = args[0]
            try:
                return f(*args)
            except Exception:
                if wrapped_class.js_emulator.session_expired():
                    wrapped_class.js_emulator.login()
                if wrapped_class.emulator.session_expired():
                    wrapped_class.core_emulator.login()
                return f(*args)
        return wraps(f)(wrapped)
    return inner



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle assignment grading for lecturers.
"""

import csv

from moodlefuse.helpers import get_cache_path_based_on_location
from moodlefuse.moodle.assignments import GRADES_FILENAME
from moodlefuse.moodle import assignments
from moodlefuse.core import config


class AssignmentGrader(object):

    def __init__(self):
        pass

    def _get_csv_path(self, location):
        if 'DOWNLOADS' in config:
            return config['DOWNLOADS'] + '/' + GRADES_FILENAME
        else:
            return get_cache_path_based_on_location(location)

    def left_format_entry(self, entry, number):
        return str(entry.ljust(number))

    def add_row(self, writer, left, middle, right):
        writer.writerow(
            (
                self.left_format_entry(left, 25),
                self.left_format_entry(middle, 35),
                self.left_format_entry(right, 10)
            )
        )

    def format_csv(self, location):
        csv_path = self._get_csv_path(location)
        with open(csv_path, 'wt') as f:
            writer = csv.writer(f)
            self.add_row(
                writer,
                assignments.GRADING_HEADER_NAME,
                assignments.GRADING_HEADER_EMAIL,
                assignments.GRADING_HEADER_GRADE
            )
            f.close()

        return csv_path

    def read_csv_grades(self, location):
        grades = []
        csv_path = self._get_csv_path(location)
        with open(csv_path, 'rt') as f:
            reader = csv.reader(f)
            for row in list(reader)[1:]:
                grades.append([element.strip() for element in row])

        return grades

    def _get_grade_from_row(self, row):
        grade = row[2]
        if assignments.GRADING_BLANK_GRADE == str(grade):
            return assignments.GRADING_BLANK_GRADE
        if assignments.GRADING_SLASH_DELIMINATOR in str(grade):
            grade = grade.split(assignments.GRADING_SLASH_DELIMINATOR)
            grade = grade[0]

        return int(float(grade))

    def get_modified(self, location, old_grades):
        new = self.read_csv_grades(location)
        old = zip(old_grades[0], old_grades[1], old_grades[2])
        changed_grades = []
        for old_row, new_row in zip(old, new):
            if self._get_grade_from_row(old_row) != self._get_grade_from_row(new_row):
                changed_grade = new_row
                changed_grade[2] = self._get_grade_from_row(new_row)
                changed_grades.append(changed_grade)

        return changed_grades

    def add_user_information(self, csv_path, grading_info):
        with open(csv_path, 'a') as f:
            writer = csv.writer(f)
            self.add_user_rows(writer, grading_info)
            f.close()

        return csv_path

    def add_user_rows(self, writer, user_grading_info):
        for name, email, grade in \
                zip(user_grading_info[0], user_grading_info[1], user_grading_info[2]):
            self.add_user_row(writer, name, email, grade)

    def add_user_row(self, writer, name, email, grade):
        self.add_row(writer, name, email, grade)



#!/usr/bin/env python
# encoding: utf-8

PARENT = '..'
THIRD_PARENT = '../../..'
CATEGORY = '//li[@aria-label="{0}"]'
GRADE_BOX = './/input[@class="quickgrade"]'
UPLOAD = '//div[@class="fp-btn-add"]'
DELETE = './/span[contains(text(), "Delete")]'
EDIT_BUTTON = './/a[contains(text(), "Edit")]'
CONTINUE = '//input[@type="submit"][@value="Continue"]'
SETTINGS = './/span[contains(text(), "Edit settings")]'
EDIT_ON = '//* [@type="submit"][@value="Turn editing on"]'
EDIT_OFF = '//* [@type="submit"][@value="Turn editing off"]'
FILE = '//span[contains(text(), "File") and @class="typename"]'
RESOURCE = '//li[@aria-label="{0}"]//span[contains(text(), "{1}")]'
USER_ROW = '//td[@class="cell c3 email" and contains(text(), "{0}")]'
QUICK_GRADING_OPTION = '//input[@name="quickgrading" and @class="ignoredirty"]'



#!/usr/bin/env python
# encoding: utf-8

"""MoodleFUSE filesystem initialization
"""

from fuse import FUSE

from moodlefuse.filesystem.file_operations import FileOperationOverrider


class Filesystem(object):

    def __init__(self, mount, testing=False):
        if not testing:
            operations_control = FileOperationOverrider(mount)
            FUSE(operations_control, mount, foreground=True)
            operations_control.end_operations()



#!/usr/bin/env python
# encoding: utf-8

"""Class to spin up a javscript enabled browser to handle Moodle
   interaction
"""

import time

from selenium import webdriver
from xvfbwrapper import Xvfb

from moodlefuse.moodle.emulator.emulator import Emulator
from moodlefuse.moodle.resources import resource_errors
from moodlefuse.exception import throws_moodlefuse_error
from moodlefuse.moodle.courses import course_errors
from moodlefuse.moodle import exception, paths
from moodlefuse.moodle import attributes
from moodlefuse.core import config

_JS_TIMEOUT = .5


class JsEmulator(Emulator):

    def __init__(self, username, password):
        super(JsEmulator, self).__init__(username, password)
        self.setup_emulator()

    def setup_emulator(self):
        if config['DEBUG'] is False:
            self.vdisplay = Xvfb(width=1280, height=720)
            self.vdisplay.xvfb_cmd.append('-noreset')
            self.vdisplay.xvfb_cmd.append('-ac')
            self.vdisplay.start()
        self.driver = webdriver.Firefox()

    def open_add_resource_menu(self, category):
        element = self.driver.find_element_by_xpath(paths.CATEGORY.format(category))
        resource_menu = element.find_element_by_class_name(attributes.SECTION)
        resource_menu.click()

    def open_edit_resource_menu(self, category, resource_name):
        xpath = paths.RESOURCE.format(category, resource_name)
        element = self.driver.find_element_by_xpath(xpath)
        element = element.find_element_by_xpath(paths.THIRD_PARENT)
        element.find_element_by_xpath(paths.EDIT_BUTTON).click()
        element.find_element_by_xpath(paths.SETTINGS).click()

    @throws_moodlefuse_error(resource_errors.UnableToRemoveFile)
    def delete_resource(self, category, resource_name):
        xpath = paths.RESOURCE.format(category, resource_name)
        element = self.driver.find_element_by_xpath(xpath)
        element = element.find_element_by_xpath(paths.THIRD_PARENT)
        element.find_element_by_xpath(paths.EDIT_BUTTON).click()
        element.find_element_by_xpath(paths.DELETE).click()
        alert = self.driver.switch_to_alert()
        alert.accept()

    @throws_moodlefuse_error(resource_errors.UnableToRenameFile)
    def rename_file(self, category, old_name, new_name):
        self.open_edit_resource_menu(category, old_name)
        self.rename_file_from_edit_screen(new_name)
        self.driver.find_element_by_id(attributes.SUBMIT2_ID).click()

    def click_option(self, select, desired_option):
        element = self.driver.find_element_by_id(select)
        for option in element.find_elements_by_tag_name(attributes.OPTION):
            if option.text == desired_option:
                option.click()

    def filter_assignments(self):
        self.click_option(attributes.FILTER, 'No filter')
        self.click_option(attributes.PERPAGE, '100')
        self.driver.find_element_by_xpath(paths.QUICK_GRADING_OPTION).click()

    def unfilter_assignments(self):
        self.driver.find_element_by_xpath(paths.QUICK_GRADING_OPTION).click()

    def grade_assignments(self, grades):
        self.filter_assignments()
        for grade in grades:
            element = self.driver.find_element_by_xpath(paths.USER_ROW.format(grade[1]))
            element = element.find_element_by_xpath(paths.PARENT)
            element = element.find_element_by_xpath(paths.GRADE_BOX)
            element.send_keys(webdriver.common.keys.Keys.CONTROL, 'a')
            element.send_keys(str(grade[2]))
            element.send_keys(webdriver.common.keys.Keys.RETURN)
            time.sleep(_JS_TIMEOUT)
            self.driver.find_element_by_xpath(paths.CONTINUE).click()

        self.unfilter_assignments()

    def rename_file_from_edit_screen(self, new_name):
        time.sleep(_JS_TIMEOUT)
        element = self.enter_text_into_textbox(attributes.NAME_ID, new_name)
        element.send_keys(webdriver.common.keys.Keys.TAB)
        element = self.driver.switch_to.active_element
        element.click()
        element.send_keys(webdriver.common.keys.Keys.CONTROL, 'a')
        element.send_keys(new_name)

    @throws_moodlefuse_error(resource_errors.UnableToAddFile)
    def add_resource(self, resource_name, resource_path):
        self.driver.find_element_by_xpath(paths.FILE).click()
        self.driver.find_element_by_name(attributes.SUBMIT).click()
        self.rename_file_from_edit_screen(resource_name)
        self.upload_resource(resource_path)

    @throws_moodlefuse_error(resource_errors.UnableToGetLocalFile)
    def upload_resource(self, resource_path):
        self.driver.find_element_by_xpath(paths.UPLOAD).click()
        time.sleep(_JS_TIMEOUT)
        element = self.driver.find_element_by_css_selector(attributes.FILE)
        element.send_keys(resource_path)
        self.driver.find_element_by_class_name(attributes.UPLOAD).click()
        self.driver.find_element_by_id(attributes.SUBMIT2_ID).click()

    @throws_moodlefuse_error(resource_errors.UnableToModifyFile)
    def edit_resource_content(self, resource_path):
        time.sleep(_JS_TIMEOUT)
        self.driver.find_element_by_class_name(attributes.FILE_CONTENT).click()
        self.driver.find_element_by_class_name(attributes.DELETE_FILE).click()
        self.driver.find_element_by_class_name(attributes.CONFIRM).click()
        self.upload_resource(resource_path)

    @throws_moodlefuse_error(course_errors.UnableToOAddCourseCategory)
    def change_most_recent_categoryname(self, new_name):
        self.check_form_checkbox(attributes.DEFAULT_NAME_ID)
        self.enter_text_into_textbox(attributes.NAME_ID, new_name)
        self.close_form()

    @throws_moodlefuse_error(exception.LoginException)
    def login(self):
        self.open_login_page(self.driver.get)
        element = self.driver.find_element_by_id(attributes.USERNAME_ID)
        element.send_keys(self.username)
        element = self.driver.find_element_by_id(attributes.PASSWORD_ID)
        element.send_keys(self.password)
        self.driver.find_element_by_id(attributes.LOGIN_ID).click()

    def open_link(self, url):
        self.driver.get(url)

    def check_form_checkbox(self, checkboxid):
        if self.driver.find_element_by_id(checkboxid).is_selected():
            self.driver.find_element_by_id(checkboxid).click()

    def enter_text_into_textbox(self, textbox, text):
        element = self.driver.find_element_by_id(textbox)
        element.clear()
        element.send_keys(text)
        return element

    @throws_moodlefuse_error(exception.UnableToToggleEditing)
    def turn_course_editing_on(self):
        element = self.driver.find_element_by_xpath(paths.EDIT_ON)
        element.click()

    @throws_moodlefuse_error(exception.UnableToToggleEditing)
    def turn_course_editing_off(self):
        element = self.driver.find_element_by_xpath(paths.EDIT_OFF)
        element.click()

    def close_form(self):
        self.driver.find_element_by_id(attributes.SUBMIT_ID).click()

    def close(self):
        self.driver.quit()
        if config['DEBUG'] is False:
            self.vdisplay.stop()



#!/usr/bin/env python
# encoding: utf-8

DIV = 'div'
LINK = 'a'
LIST = 'li'
TABLE = 'td'
SPAN = 'span'
CLASS = 'class'
TITLE = 'title'
HEADING3 = 'h3'
LINKTEXT = 'href'
LABEL = 'aria-label'

ENCODING = 'utf-8'

OPTION = 'option'
NAME_ID = 'id_name'
LOGIN_ID = 'loginbtn'
USERNAME_ID = 'username'
PASSWORD_ID = 'password'
SUBMIT_ID = 'id_submitbutton'
SUBMIT2_ID = 'id_submitbutton2'
DEFAULT_NAME_ID = 'id_usedefaultname'

LOGIN = 'login'
FILTER = 'id_filter'
MODULES = 'mymodules'
PERPAGE = 'id_perpage'
FIRST_CELL = 'cell c0'
THIRD_CELL = 'cell c2'
FIFTH_CELL = 'cell c5'
SUBMIT = 'submitbutton'
UPLOAD = 'fp-upload-btn'
FILE = 'input[type="file"]'
FILE_CONTENT = 'fp-mainfile'
INSTANCE = 'activityinstance'
CONFIRM = 'fp-dlg-butconfirm'
EDIT_SUMMARY = 'Edit summary'
DELETE_FILE = 'fp-file-delete'
INSTANCE_NAME = 'instancename'
COURSE_CONTENT = 'course-content'
THIRD_CELL_EMAIL = 'cell c3 email'
SECTION = 'section-modchooser-text'
INCREASE_SECTIONS = 'increase-sections'
COURSES = 'courses frontpage-course-list-all'



#!/usr/bin/env python
# encoding: utf-8

"""Filesystem operations classs, overrides FUSE operations
"""

import errno
import os

from fuse import FuseOSError, Operations

from moodlefuse.filesystem.file_system_translator import FileSystemTranslator
from moodlefuse.filesystem.path_parser import PathParser


class FileOperationOverrider(Operations):

    def __init__(self, root):
        self.root = os.path.realpath(root)
        self.translator = FileSystemTranslator()

    def end_operations(self):
        self.translator.close_browsers()

    def __call__(self, op, path, *args):
        return super(FileOperationOverrider, self).__call__(
            op, self.root + path, *args
        )

    def access(self, path, mode):
        if not self.translator.path_exists_in_moodle(path):
            raise FuseOSError(errno.EACCES)

    def create(self, path, mode, fi=None):
        cache_path = self.translator.create_file(path)
        if cache_path is not None:
            return os.open(cache_path, os.O_WRONLY)
        else:
            return 1

    def flush(self, path, fh):
        pass

    def fsync(self, path, fdatasync, fh):
        pass

    def getattr(self, path, fh=None):
        if not self.translator.path_exists_in_moodle(path):
            raise FuseOSError(errno.ENOENT)
        else:
            return self.translator.get_file_attributes(path)

    def mkdir(self, path, mode):
        self.translator.make_directory(path)

    def open(self, path, flags):
        cache_path = self.translator.open_file(path)
        if cache_path is not None:
            return os.open(cache_path, flags)
        else:
            return 1

    def read(self, path, length, offset, fh):
        os.lseek(fh, offset, os.SEEK_SET)
        return os.read(fh, length)

    def readdir(self, path, fh):
        dirents = self.translator.get_directory_contents_based_on_path(path)
        for r in dirents:
            yield r

    def release(self, path, fh):
        return os.close(fh)

    def rename(self, old, new):
        self.translator.rename_file(old, new)

    def rmdir(self, path):
        pass

    def statfs(self, path):
        return dict(f_bsize=512, f_blocks=4096, f_bavail=2048)

    def truncate(self, path, length, fh=None):
        cache_path = self.translator.open_file(path)
        with open(cache_path, 'r+') as cache_file:
            cache_file.truncate(length)

    def unlink(self, path):
        self.translator.remove_resource(path)

    def utimens(self, path, times=None):
        pass

    def write(self, path, buf, offset, fh):
        os.lseek(fh, offset, os.SEEK_SET)
        location = PathParser.get_position_in_filesystem_as_array(path)
        response = os.write(fh, buf)
        if PathParser.is_file(location):
            self.translator.modify_file(path)
        elif PathParser.is_assignment(location):
            response = os.write(fh, buf)

        return response

    link = None

    chmod = None

    chown = None

    mknod = None

    readlink = None

    symlink = None



#!/usr/bin/env python
# encoding: utf-8

"""Class to setup MoodleFUSE
"""

import argparse
import sys
import os

from ConfigParser import SafeConfigParser

from moodlefuse.helpers import get_password_for_user
from moodlefuse.model_manager import setup_model

config = {}


def setup(settings=None):
    if settings is not None:
        from_object(settings)
    else:
        args = _generate_args()
        profile = args.pop('profile')
        config['DEBUG'] = args.pop('debug')
        load_database()
        config_file = _load_config_file()
        _config_from_config_profile(config_file, profile)


def load_database():
    database_file = os.path.join(
        os.path.expanduser('~'),
        '.moodlefuse/moodlefuse.sqlite'
    )
    setup_model('sqlite:///' + database_file)


def from_object(obj):
    for key in dir(obj):
        if key.isupper():
            config[key] = getattr(obj, key)


def _generate_args():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-p',
        '--profile',
        required=False,
        help='The profile to run moodlefuse with, default is initial',
        default='initial'
    )

    parser.add_argument(
        '-d',
        '--debug',
        required=False,
        help='Turn debug on for application',
        default=False
    )

    args = parser.parse_args()

    return vars(args)


def _load_config_file():
    config_file = os.path.join(
        os.path.expanduser('~'),
        '.moodlefuse/moodlefuse.conf'
    )

    if not os.path.exists(config_file):
        sys.exit('No configuration found, please run moodlefuse-configure')

    return config_file


def _config_doesnt_exist(profile):
    sys.exit(
        'No profile matching ' +
        profile +
        ' found in configuration, please run moodlefuse-configure -p ' +
        profile
    )


def _save_config(configuration, profile):
    for attribute in configuration.options(profile):
        if attribute == 'local_moodle_folder':
            config[attribute.upper()] = str(
                os.path.join(
                    os.path.expanduser('~'),
                    configuration.get(
                        profile,
                        attribute
                    )
                ))
        else:
            config[attribute.upper()] = configuration.get(profile, attribute)


def _config_from_config_profile(config_file, profile):
    configuration = SafeConfigParser()
    configuration.read(config_file)

    if not configuration.has_section(profile):
        _config_doesnt_exist(profile)

    config['PASSWORD'] = get_password_for_user(
        configuration.get(
            profile,
            'username'
        )
    )

    _save_config(configuration, profile)



#!/usr/bin/env python
# encoding: utf-8

"""Class to handle Exceptions relating to MoodleFUSE actions
"""

from functools import wraps

from moodlefuse.core import config


class MoodleFuseException(Exception):
    def __init__(self, debug_info):
        exception_reason = "ERROR ENCOUNTERED: MoodleFUSE has encountered an error."
        debug_info = debug_info
        self.message = exception_reason + debug_info

    def __str__(self):
        return self.message


def throws_moodlefuse_error(moodlefuse_error):
    def inner(f):
        def wrapped(*args):
            try:
                return f(*args)
            except Exception, e:
                if config['DEBUG'] is False:
                    raise moodlefuse_error()
                else:
                    raise e
        return wraps(f)(wrapped)
    return inner



#!/usr/bin/env python
# encoding: utf-8

from tests import MoodleFuseAppTestCase
from tests.data.settings import DOWNLOADS

import os


class FilesystemTestCase(MoodleFuseAppTestCase):

    def cache_created_test(self):
        self.assertTrue(os.path.exists(DOWNLOADS))

    def mount_created_test(self):
        self.assertTrue(os.path.exists(self.filesystem_root))

    def flush_test(self):
        self.ops.flush('', None)

    def fsync_test(self):
        self.ops.fsync('', None, None)

    def utimens_test(self):
        self.ops.utimens('')

    def unlink_test(self):
        self.ops.unlink('')

    def remdir_test(self):
        self.ops.rmdir('')

    def ops_test(self):
        self.ops.__call__('flush', '/', {})

    def stat_test(self):
        self.assertEquals(
            self.ops.statfs(''),
            dict(f_bsize=512, f_blocks=4096, f_bavail=2048)
        )



#!/usr/bin/env python
# encoding: utf-8

"""Derived class to parse Moodle resource items, it parses
   Moodle and therefor extends off of Parser
"""

from moodlefuse.moodle.parser import Parser
from moodlefuse.moodle import attributes
from moodlefuse.moodle import resources


class ResourceParser(Parser):

    def __init__(self):
        super(ResourceParser, self).__init__()

    def parse_course_resources(self, category_content):
        resources_html = self._get_resource_html(category_content)
        return self.scraper.get_instances_from_span_list(resources_html)

    def parse_course_resource_url(self, category_content, filename):
        activity_instances = self.scraper.get_html_items_with_divclass(
            category_content, attributes.INSTANCE)

        return self.scraper.get_link_from_span_list_with_type_and_name(
            activity_instances, resources.FILE, filename
        )

    def parse_course_assignment_url(self, category_content, filename):
        activity_instances = self.scraper.get_html_items_with_divclass(
            category_content, attributes.INSTANCE)

        return self.scraper.get_link_from_span_list_with_type_and_name(
            activity_instances, resources.ASSIGNMENT, filename
        )

    def _get_resource_html(self, category_content):
        return self.scraper.get_html_items_with_spanclass(
            category_content, attributes.INSTANCE_NAME)



#!/usr/bin/env python
# encoding: utf-8

"""Base class to parse Moodle items
"""

from moodlefuse.moodle.scraper import Scraper
from moodlefuse.moodle import attributes


class Parser(object):

    def __init__(self):
        self.scraper = Scraper()

    def remove_special_chars(self, string):
        return string.replace(u'\xa0', '')

    def remove_unicode(self, items):
        return [item.encode(attributes.ENCODING) for item in items]

    def get_link_from_item(self, item):
        if item is not None:
            return item[attributes.LINKTEXT]

        return None
